# Copyright (C) 2013 Team tiramisu (see AUTHORS for all contributors)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# The original `Config` design model is unproudly borrowed from
# the rough gus of pypy: pypy: http://codespeak.net/svn/pypy/dist/pypy/config/
# the whole pypy projet is under MIT licence
# ____________________________________________________________

"""Storage connections, executions and managements.

Storage is basic components used to set informations in DB
"""


from time import time
from tiramisu.error import ConfigError
from tiramisu.i18n import _


class StorageType(object):
    default_storage = 'dictionary'
    storage_type = None
    mod = None

    def set(self, name):
        if self.storage_type is not None:
            raise ConfigError(_('storage_type is already set, cannot rebind it'))
        self.storage_type = name

    def get(self):
        if self.storage_type is None:
            self.storage_type = self.default_storage
        storage = self.storage_type
        if self.mod is None:
            modulepath = 'tiramisu.storage.{0}'.format(storage)
            mod = __import__(modulepath)
            for token in modulepath.split(".")[1:]:
                mod = getattr(mod, token)
            self.mod = mod
        return self.mod


storage_type = StorageType()


def set_storage(name, **args):
    storage_type.set(name)
    settings = storage_type.get().Setting()
    for option, value in args.items():
        try:
            getattr(settings, option)
            setattr(settings, option, value)
        except AttributeError:
            raise ValueError(_('option {0} not already exists in storage {1}'
                               '').format(option, name))


def get_storage(context, session_id, persistent):
    def gen_id(config):
        return str(id(config)) + str(time())

    if session_id is None:
        session_id = gen_id(context)
    imp = storage_type.get()
    storage = imp.Storage(session_id, persistent)
    return imp.Settings(storage), imp.Values(storage)


def list_sessions():
    return storage_type.get().list_sessions()


def delete_session(session_id):
    return storage_type.get().delete_session(session_id)


#__all__ = (,)
